/**
 * Based on examples in
 *     Head First Object-Oriented Analysis and Design
 *     by Brett D. McLaughlin, Gary Pollice, and David West
 *     O'Reilly, 2007
 *     ISBN 978-0-596-00867-3
 *
 * Ported from Java by
 *     Ronald Mak
 *     Department of Computer Engineering
 *     San Jose State University
 *     San Jose, CA  95192
 */
#ifndef INSTRUMENTSPEC_H_
#define INSTRUMENTSPEC_H_

#include <string>
#include "Builder.h"
#include "Type.h"
#include "Wood.h"

using namespace std;

class InstrumentSpec
{
public:
    InstrumentSpec(Builder builder, string model, Type type,
                   Wood back_wood, Wood top_wood)
        : model(model), builder(builder), type(type),
          back_wood(back_wood), top_wood(top_wood)
    {}

    virtual ~InstrumentSpec() = 0;

    Builder get_builder() const   { return builder; }
    string  get_model() const     { return model; }
    Type    get_type() const      { return type; }
    Wood    get_back_wood() const { return back_wood; }
    Wood    get_top_wood() const  { return top_wood; }

    bool matches(InstrumentSpec *other_spec);

private:
    string model;
    Builder builder;
    Type type;
    Wood back_wood, top_wood;

    string to_lower(string str);
};

inline InstrumentSpec::~InstrumentSpec() {}

inline string InstrumentSpec::to_lower(string str)
{
    transform(str.begin(), str.end(), str.begin(), ::tolower);
    return str;
}

#endif /* INSTRUMENTSPEC_H_ */
